/*
 * file BrowserDataModel.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.BrowserDataModel
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.stp.client.samples;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicBorders;
import javax.swing.table.AbstractTableModel;
import javax.wvcm.WvcmException;

/**
 * An all-in-one class for displaying a table of browser data
 */
public abstract class BrowserDataModel
    extends AbstractTableModel
{
    /**
     * Returns the object that is viewable at a given index in the table model
     *
     * @param  index  The index into the table model of the item to show
     *
     * @return  null if the item is not viewable; otherwise an object that
     * is viewable.
     */
    abstract Object getViewable(int index);

    /**
     * Displays an object returned by getViewable
     *
     * @param  viewable  The object to display.
     */
    abstract void show(Object viewable) throws WvcmException;

    /**
     * Displays the content of the object whose properties are displayed in 
     * this table.
     * @throws Throwable If the content cannot be displayed.
     */
    void showContent() throws Throwable {} 
    
    /**
     * Returns the label to place on the Hide/Show Errors button
     * @return If null or empty, the Hide/Show button is not displayed; 
     * otherwise the button will be displayed with the given label.
     */
    String toggleErrorsLabel() { return ""; }
    
    /**
     * Rereads the data from the resource and redisplays it
     * 
     * @throws WvcmException If problems arise during the process. 
     */
    void redisplay() throws WvcmException {}

    /**
     * Processes the Show/Hide Errors button when clicked.
     * @throws WvcmException if the action cannot be carried out.
     */
    void toggleErrors() throws WvcmException {}
    
    /**
     * A specification for the interface between the BrowserDataModel and a
     * provider of operations to be applied to the object displayed in the
     * model.
     */
    interface Operations {
	    /**
	     * Returns a list of operations that can be applied to the object displayed
	     * by this data model.
	     * If not null, the list is placed in a combo-box from which the user
	     * may select and execute an operation.
	     * @return A list of possible operations for the resource. If null, no
	     * operations combo-box will be displayed.
	     */
	    Vector getList();
	    
	    /**
	     * Performs the operation identified by the argument to the object displayed
	     * by this data model.
	     * @param op One of the operations supplied in the list returned by
	     * resourceOperations.
	     */
	    void execute(Object op, BrowserDataModel model);
    }
    
    Operations getOperationsObject() { return null; }
    
    /**
     * Generates a display of this table.
     * @param  title  The window title
     * @param  hasContent true if the ShowContent button should be displayed
     */
    JFrame showModel(
        String       title,
        boolean      hasContent)
    {
        final JFrame frame = new JFrame(title);
        // Comment this line when using Sun's TableSorter class
        final JTable table = new JTable(this);
        // Uncomment these lines when using Sun's TableSorter class
        // final TableSorter sorter = (m_sorter = new TableSorter(this));
        // final JTable      table = new JTable(sorter);
        //         
        // sorter.setTableHeader(table.getTableHeader());
        // sorter.setSortingStatus(0, TableSorter.ASCENDING);
        //
        final JPanel  panel = new JPanel(new BorderLayout());
        final JPanel  subpanel = new JPanel(new FlowLayout());
        final JButton button = new JButton("View");

        subpanel.add(button, BorderLayout.SOUTH);
        button.setEnabled(false);
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0)
                {
                    int[] selected = selectedRows(table);

                    for (int i = 0; i < selected.length; ++i) {
                        try {
                            show(getViewable(selected[i]));
                        } catch (WvcmException ex) {
                            JOptionPane.showMessageDialog(null, ex.getLocalizedMessage());
                        }
                    }
                }
            });

        if (hasContent) {
            JButton show = new JButton("Show Content");
            subpanel.add(show);
            show.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0)
                {
                    try {
                        showContent();
                    } catch (Throwable ex) {
                        JOptionPane.showMessageDialog(null, ex.getLocalizedMessage());
                    }
                }
            });
        }
        
        String label = toggleErrorsLabel();
        
        if (label.length() > 0) {
            JButton show = new JButton(label);
            subpanel.add(show);
            show.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0)
                {
                    try {
                        toggleErrors();
                    } catch (Throwable ex) {
                        JOptionPane.showMessageDialog(null, ex.getLocalizedMessage());
                    }
                }
            });
        }
        
        final Operations ops = getOperationsObject();
        
        if (ops != null) {
        	Vector list = ops.getList();
        	
        	if (list != null && !list.isEmpty()) {
	        	final JComboBox box = new JComboBox(list);
	            JButton exec = new JButton("<<GO!");
	            subpanel.add(box);
	            subpanel.add(exec);
	            exec.addActionListener(new ActionListener() {
	                public void actionPerformed(ActionEvent arg0)
	                {
	                    ops.execute(box.getSelectedItem(), BrowserDataModel.this);
	                }
	            });
        	}
        }
        
        JButton exit = new JButton("Exit");
        subpanel.add(exit);
        exit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                System.exit(0);
            }
        });
        
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        // Ask to be notified of selection changes.
        ListSelectionModel rowSM = table.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e)
                {
                    if (!e.getValueIsAdjusting()) {
                        int[] selected = selectedRows(table);
                        button.setEnabled(false);

                        for (int i = 0; i < selected.length; ++i) {
                            if (getViewable(selected[i]) != null) {
                                button.setEnabled(true);

                                break;
                            }
                        }
                    }
                }
            });

        panel.add(new JScrollPane(table), BorderLayout.CENTER);
        panel.add(subpanel, BorderLayout.SOUTH);
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.setContentPane(panel);
        frame.setBounds(lastX += 3, lastY += 3, 
        				600, 17*(5 + table.getRowCount()));
        frame.addWindowFocusListener(new WindowAdapter(){
            public void windowGainedFocus(
                WindowEvent arg0)
            {
                super.windowGainedFocus(arg0);
                lastX = arg0.getWindow().getX();
                lastY = arg0.getWindow().getY();
            }
        });

        frame.setVisible(true);
        
        return m_frame = frame;
    }
    
    /**
     * Returns the rows of the table currently selected in the display
     * @param table The table
     * @return An int[] containing the index of each row that is highlighted
     */
    int[] selectedRows(JTable table)
    {
        int[] selected = table.getSelectedRows();

        // Uncomment these lines when using Sun's TableSorter class
        // for (int I = 0; I < selected.length; ++I)
        //      selected[I] = m_sorter.modelIndex(selected[I]);
        //
        
        return selected;
    }
    
    /**
     * Displays the content of file in a window.
     * @param title The title string to appear in the window banner
     * @param file A File identifying the file whose content is to be displayed
     * @return A JFrame for the displayed window.
     * @throws Throwable If the content of the file cannot be displayed.
     */
    static JFrame showFile(
        String title, 
        File file) throws Throwable
    {
        JTextArea text = new JTextArea(680, 400);
        FileReader reader = new FileReader(file);
        char[] buf = new char[1000];
        int n;
    
        while ((n=reader.read(buf)) > 0) {
            text.append(new String(buf,0,n));
        }
        
        reader.close();
        file.delete();
        
        text.setBorder(BasicBorders.getTextFieldBorder());
        text.setCaretPosition(0);
        text.setFont(new Font("Monospaced", Font.PLAIN, 12));
        
        JFrame frame = new JFrame(title);
        frame.setContentPane(new JScrollPane(text));
        frame.setBounds(lastX += 20, lastY += 40, 600, 300);
        frame.setVisible(true);
        
        return frame;
    }
    
    // Uncomment this line when using a TableSorter
    // private TableSorter m_sorter;
    //
    
    /** The frame containing this data model */
    protected JFrame m_frame;
    
    /** The X offset for the next window displayed */
    static int lastX = 180;

    /** The Y offset for the next window displayed */
    static int lastY = 360;
}
